/*
 * $Header$
 */

#include "all.h"

/**
 */
static int S_Fwrite (char p[], size_t length, size_t remain);

Boolean APP_isDebug = FALSE;
size_t APP_count = 1024 * 1024;
size_t APP_io_chunk = 1024;

static FILE *S_fp = NULL;

int
APP_Init (void)
{
  int rc = 0;

  srand (time (NULL));
  return rc;
}

void
APP_Uninit (void)
{
}

int
APP_ParseCommandLine (int argc, char *argv[])
{
  int rc = 0;
  int c;
  unsigned u;

  while (rc == 0 && (c = getopt (argc, argv, "gi:s:")) != EOF) {
    switch (c) {
    case 'g':
      APP_isDebug = TRUE;
      break;

    case 'i':
      if (sscanf (optarg, "%u", &u) == 1) {
	APP_io_chunk = u;
	if (APP_isDebug) {
	  fprintf (stderr, "%s:%d: io_chunk is %lu characters\n", __FILE__,
		   __LINE__, (unsigned long) APP_io_chunk);
	}
      } else {
	fprintf (stderr, "%s:%d: ", __FILE__, __LINE__);
	fprintf (stderr, "can't parse \"%s\" into an ", optarg);
	fprintf (stderr, "unsigned, integral number of characters\n");
	rc = -56;
      }
      break;

    case 's':
      if (sscanf (optarg, "%u", &u) == 1) {
	APP_count = u;
	if (APP_isDebug) {
	  fprintf (stderr, "%s:%d: count is %lu characters\n", __FILE__,
		   __LINE__, (unsigned long) APP_count);
	}
      } else {
	fprintf (stderr, "%s:%d: ", __FILE__, __LINE__);
	fprintf (stderr, "can't parse \"%s\" into an ", optarg);
	fprintf (stderr, "unsigned, integral number of characters\n");
	rc = -56;
      }
      break;

    case '?':
    case ':':
      rc = -48;
      break;

    default:
      fprintf (stderr, "%s:%d: unexpected value from getopt, '%c'\n",
	       __FILE__, __LINE__, c);
      rc = -49;
    }
  }
  return rc;
}

int
APP_Loop (size_t count, char p[], size_t plen,
          int (*sendfn) (char[], size_t, size_t))
{
  int rc = 0;
  size_t length;

  assert (p != NULL);
  assert (plen > 0);
  assert (sendfn != NULL);
  while (rc == 0 && count > 0) {
    length = MIN (count, plen);
    count -= length;
    G_Fill (p, length);
    if ((*sendfn) (p, length, count) != 0) {
      fprintf (stderr, "%s:%d: *sendfn: failed\n", __FILE__, __LINE__);
      rc = -42;
    }
  }
  return rc;
}

int
APP_LoopFile (size_t count, FILE * fp)
{
  int rc = 0;
  char *p;

  assert (count > 0);
  assert (fp != NULL);
  if (S_fp == NULL) {
    p = malloc (APP_io_chunk);
    if (p != NULL) {
      S_fp = fp;
      if (APP_Loop (count, p, APP_io_chunk, &S_Fwrite) != 0) {
	fprintf (stderr, "%s:%d: failure\n", __FILE__, __LINE__);
	rc = 2;
      }
      free (p);
    } else {
      fprintf (stderr, "%s:%d: malloc: %s\n", __FILE__, __LINE__,
	       strerror (errno));
      rc = 3;
    }
    S_fp = NULL;
  } else {
    fprintf (stderr, "%s:%d: calls to APP_LoopFile must not nest\n",
	     __FILE__, __LINE__);
    rc = 3;
  }
  return rc;
}

static int
S_Fwrite (char p[], size_t length, size_t remain)
{
  int rc = 0;

  assert (p != NULL);
  assert (length > 0);
  assert (S_fp != NULL);
  if (fwrite (p, length, 1, S_fp) != 1) {
    fprintf (stderr, "%s:%d: fwrite: %s\n", __FILE__, __LINE__,
	     strerror (errno));
    rc = -13;
  }
  return rc;
}

/* --- end of file --- */
