/*
 * $Header$
 */

#include "all.h"

static int S_Init (void);
static int S_Msgsnd (char a[], size_t length, size_t remain);
static void S_Uninit (void);

/**
 * The identifier for our message queue.
 */
static int S_qid;

/**
 * The single, dynamically allocated message we use for the
 * message queue.
 */
static struct MSGQ_Message *S_msg;

/*--------------------------------------------------------------------*/

int
main (int argc, char *argv[])
{
  int rc = 0;

  if (S_Init () == 0) {
    if (APP_ParseCommandLine (argc, argv) == 0) {
      S_msg = malloc (MSGQ_Size (APP_io_chunk));
      if (S_msg != NULL) {
	if (APP_Loop (APP_count, S_msg->a, APP_io_chunk, &S_Msgsnd) == 0) {
	  /*
	   * success
	   */
	} else {
	  fprintf (stderr, "%s:%d: failure\n", __FILE__, __LINE__);
	  rc = 5;
	}
	free (S_msg);
      } else {
	fprintf (stderr, "%s:%d: malloc: %s\n", __FILE__, __LINE__,
		 strerror (errno));
	rc = 5;
      }
    } else {
      fprintf (stderr, "%s:%d: failure\n", __FILE__, __LINE__);
      rc = -45;
    }
    S_Uninit ();
  } else {
    fprintf (stderr, "%s:%d: failure\n", __FILE__, __LINE__);
    rc = -43;
  }
  return rc == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
}

static int
S_Init (void)
{
  int rc = 0;

  if (APP_Init () == 0) {
    S_qid = msgget (MSGQ_KEY, IPC_R | IPC_W | /* IPC_M | */ IPC_CREAT);
    if (S_qid != -1) {
      /*
       * The message queue was created.  Or maybe it already existed.
       */
    } else {
      fprintf (stderr, "%s:%d: msgget: %s\n", __FILE__, __LINE__,
	       strerror (errno));
      rc = 13;
    }
  } else {
    fprintf (stderr, "%s:%d: failure\n", __FILE__, __LINE__);
    rc = 17;
  }
  return rc;
}

static int
S_Msgsnd (char a[], size_t length, size_t remain)
{
  int rc = 0;

  assert (a == S_msg->a);
  assert (length > 0);
  S_msg->mtype = 1;
  S_msg->count = length;
  S_msg->isEnd = remain == 0;
  if (msgsnd (S_qid, S_msg, MSGQ_Size (APP_io_chunk), 0) != 0) {
    fprintf (stderr, "%s:%d: msgsnd: %s\n", __FILE__, __LINE__,
	     strerror (errno));
    rc = 6;
  }
  return rc;
}

static void
S_Uninit (void)
{
  APP_Uninit ();
}

/* --- end of file --- */
