/*
 * $Header$
 */

#include "all.h"

static int S_Init (void);
static void S_Uninit (void);
static int S_Run (void);

/**
 * The identifier for our message queue.
 */
static int S_qid;

/**
 * The single, dynamically allocated message we use for the
 * message queue.
 */
static struct MSGQ_Message *S_msg;

/*--------------------------------------------------------------------*/

int
main (int argc, char *argv[])
{
  int rc = 0;

  if (S_Init () == 0) {
    if (APP_ParseCommandLine (argc, argv) == 0) {
      S_msg = malloc (MSGQ_Size (APP_io_chunk));
      if (S_msg != NULL) {
	if (S_Run () == 0) {
	  /*
	   * success
	   */
	} else {
	  fprintf (stderr, "%s:%d: failure\n", __FILE__, __LINE__);
	  rc = 5;
	}
	free (S_msg);
      } else {
	fprintf (stderr, "%s:%d: failure\n", __FILE__, __LINE__);
	rc = -45;
      }
    } else {
      fprintf (stderr, "%s:%d: failure\n", __FILE__, __LINE__);
      rc = -43;
    }
    S_Uninit ();
  } else {
    fprintf (stderr, "%s:%d: failure\n", __FILE__, __LINE__);
    rc = -11;
  }
  return rc == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
}

static int
S_Init (void)
{
  int rc = 0;

  if (APP_Init () == 0) {
    S_qid = msgget (MSGQ_KEY, IPC_R | IPC_W | /* IPC_M | */ IPC_CREAT);
    if (S_qid != -1) {
      /*
       * The message queue was created.  Or maybe it already existed.
       */
    } else {
      fprintf (stderr, "%s:%d: msgget: %s\n", __FILE__, __LINE__,
	       strerror (errno));
      rc = 13;
    }
  } else {
    fprintf (stderr, "%s:%d: failure\n", __FILE__, __LINE__);
    rc = 11;
  }
  return rc;
}

static int
S_Run (void)
{
  int rc = 0;
  size_t sz;

  sz = MSGQ_Size (APP_io_chunk);
  do {
    if (msgrcv (S_qid, S_msg, sz, 0, 0) == sz) {
      assert (rc == 0);
    } else {
      assert (rc == 0);
      fprintf (stderr, "%s:%d: msgrcv: %s\n", __FILE__, __LINE__,
	       strerror (errno));
      rc = 29;
    }
  } while (rc == 0 && !S_msg->isEnd);
  return rc;
}

static void
S_Uninit (void)
{
  APP_Uninit ();
}

/* --- end of file --- */
