/*
 * $Header$
 */

#include "all.h"

/*
 * Performance note: I originally implemented thie program with
 * a loop on 'getchar'.  A particular test required 10.11 seconds
 * to run.  Then I converted it to 'read' on a buffer larger than
 * one character.  The same test required 9.53 seconds to run.
 * So converting from 'getchar' to 'read' improved performance
 * by 5 percent.  Make of that what you will.
 */

static int S_Connect (char filename[]);

/**
 * The name of the Unix socket we create & listen on.
 */
static char *S_filename = "tmp/unix-socket.tmp";

/*--------------------------------------------------------------------*/

int
main (int argc, char *argv[])
{
  int rc = 0;
  int fd;
  char buffer[4 * 1024];

  if (APP_Init () == 0) {
    if (APP_ParseCommandLine (argc, argv) == 0) {
      fd = S_Connect (S_filename);
      if (fd != -1) {
	close (0);
	dup (fd);
	while (read (0, buffer, sizeof buffer) > 0) {
	  /*
	   * nothing
	   */
	}
      } else {
	rc = 6;
      }
    } else {
      fprintf (stderr, "%s:%d: S_ParseCommandLine failed\n", __FILE__,
	       __LINE__);
      rc = 6;
    }
    APP_Uninit ();
  } else {
    fprintf (stderr, "%s:%d: failure\n", __FILE__, __LINE__);
    rc = 10;
  }
  return rc == 0 ? EXIT_SUCCESS : EXIT_FAILURE;
}

static int
S_Connect (char filename[])
{
  int fd = -1;
  int i;
  struct sockaddr_un sau;
  int tries = 0;

  assert (filename != NULL);
  fd = socket (AF_UNIX, SOCK_STREAM, 0);
  if (fd != -1) {
    memset (&sau, 0, sizeof sau);
    /* sau.sun_len = strlen (filename) + 1; */
    sau.sun_family = AF_UNIX;
    strcpy (sau.sun_path, filename);
    while ((i = connect (fd, (struct sockaddr *) &sau, sizeof sau)) != 0 &&
	   tries < 10) {
      ++tries;
      sleep (2);
    }
    if (i != 0) {
      fprintf (stderr, "%s:%d: connect: %s\n", __FILE__, __LINE__,
	       strerror (errno));
      close (fd);
      fd = -1;
    }
  } else {
    fprintf (stderr, "%s:%d: socket: %s\n", __FILE__, __LINE__,
	     strerror (errno));
  }
  return fd;
}

/* --- end of file --- */
